@@ -1,9 +1,9 @@
 /*
  * security/ccsecurity/maymount.c
  *
- * Copyright (C) 2005-2010  NTT DATA CORPORATION
+ * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 1.7.2   2010/04/01
+ * Version: 1.7.3+   2012/04/01
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -34,7 +34,8 @@
  * @vfsmnt: Pointer to "struct vfsmount".
  * @dentry: Pointer to "struct dentry".
  *
- * Returns true if @vfsmnt is parent directory compared to @nd, false otherwise.
+ * Returns true if @vfsmnt is parent directory compared to @nd,
+ * false otherwise.
  */
 static bool ccs_conceal_mount(struct path *path, struct vfsmount *vfsmnt,
 			      struct dentry *dentry)
@@ -75,8 +76,7 @@ static int __ccs_may_mount(struct path *path)
 	struct mnt_namespace *namespace = current->nsproxy->mnt_ns;
 #endif
 	if (!namespace ||
-	    ccs_init_request_info(&r, NULL,
-				  CCS_MAX_MAC_INDEX + CCS_CONCEAL_MOUNT)
+	    ccs_init_request_info(&r, CCS_MAX_MAC_INDEX + CCS_CONCEAL_MOUNT)
 	    == CCS_CONFIG_DISABLED)
 		return 0;
 	found = false;
