@@ -1,9 +1,9 @@
 /*
  * security/ccsecurity/policy_io.c
  *
- * Copyright (C) 2005-2011  NTT DATA CORPORATION
+ * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 1.7.3   2011/04/01
+ * Version: 1.7.3+   2012/04/01
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -305,7 +305,7 @@ static void ccs_check_profile(void)
 	if (ccs_profile_version != 20090903)
 		panic("Profile version %u is not supported.\n",
 		      ccs_profile_version);
-	printk(KERN_INFO "CCSecurity: 1.7.3   2011/04/01\n");
+	printk(KERN_INFO "CCSecurity: 1.7.3+   2012/04/01\n");
 	printk(KERN_INFO "Mandatory Access Control activated.\n");
 }
 
@@ -2298,34 +2298,19 @@ int ccs_supervisor(struct ccs_request_info *r, const char *fmt, ...)
  * ccs_poll_query - poll() for /proc/ccs/query.
  *
  * @file: Pointer to "struct file".
- * @wait: Pointer to "poll_table".
+ * @wait: Pointer to "poll_table". May be NULL.
  *
  * Returns POLLIN | POLLRDNORM when ready to read, 0 otherwise.
  *
  * Waits for access requests which violated policy in enforcing mode.
  */
-static int ccs_poll_query(struct file *file, poll_table *wait)
+static unsigned int ccs_poll_query(struct file *file, poll_table *wait)
 {
-	struct list_head *tmp;
-	bool found = false;
-	u8 i;
-	for (i = 0; i < 2; i++) {
-		spin_lock(&ccs_query_list_lock);
-		list_for_each(tmp, &ccs_query_list) {
-			struct ccs_query_entry *ptr =
-				list_entry(tmp, struct ccs_query_entry, list);
-			if (ptr->answer)
-				continue;
-			found = true;
-			break;
-		}
-		spin_unlock(&ccs_query_list_lock);
-		if (found)
-			return POLLIN | POLLRDNORM;
-		if (i)
-			break;
-		poll_wait(file, &ccs_query_wait, wait);
-	}
+	if (!list_empty(&ccs_query_list))
+		return POLLIN | POLLRDNORM;
+	poll_wait(file, &ccs_query_wait, wait);
+	if (!list_empty(&ccs_query_list))
+		return POLLIN | POLLRDNORM;
 	return 0;
 }
 
@@ -2351,8 +2336,6 @@ static void ccs_read_query(struct ccs_io_buffer *head)
 	list_for_each(tmp, &ccs_query_list) {
 		struct ccs_query_entry *ptr
 			= list_entry(tmp, struct ccs_query_entry, list);
-		if (ptr->answer)
-			continue;
 		if (pos++ != head->read_step)
 			continue;
 		len = ptr->query_len;
@@ -2371,8 +2354,6 @@ static void ccs_read_query(struct ccs_io_buffer *head)
 	list_for_each(tmp, &ccs_query_list) {
 		struct ccs_query_entry *ptr
 			= list_entry(tmp, struct ccs_query_entry, list);
-		if (ptr->answer)
-			continue;
 		if (pos++ != head->read_step)
 			continue;
 		/*
@@ -2422,8 +2403,12 @@ static int ccs_write_answer(struct ccs_io_buffer *head)
 			= list_entry(tmp, struct ccs_query_entry, list);
 		if (ptr->serial != serial)
 			continue;
-		if (!ptr->answer)
-			ptr->answer = answer;
+		ptr->answer = answer;
+		/* Remove from ccs_query_list. */
+		if (ptr->answer) {
+			list_del(&ptr->list);
+			INIT_LIST_HEAD(&ptr->list);
+		}
 		break;
 	}
 	spin_unlock(&ccs_query_list_lock);
@@ -2589,19 +2574,17 @@ int ccs_open_control(const u8 type, struct file *file)
  * ccs_poll_control - poll() for /proc/ccs/ interface.
  *
  * @file: Pointer to "struct file".
- * @wait: Pointer to "poll_table".
+ * @wait: Pointer to "poll_table". May be NULL.
  *
- * Waits for read readiness.
- * /proc/ccs/query is handled by /usr/sbin/ccs-queryd and
- * /proc/ccs/grant_log and /proc/ccs/reject_log are handled by
- * /usr/sbin/ccs-auditd .
+ * Returns POLLIN | POLLRDNORM | POLLOUT | POLLWRNORM if ready to read/write,
+ * POLLOUT | POLLWRNORM otherwise.
  */
-int ccs_poll_control(struct file *file, poll_table *wait)
+unsigned int ccs_poll_control(struct file *file, poll_table *wait)
 {
 	struct ccs_io_buffer *head = file->private_data;
 	if (!head->poll)
-		return -ENOSYS;
-	return head->poll(file, wait);
+		return POLLIN | POLLRDNORM | POLLOUT | POLLWRNORM;
+	return head->poll(file, wait) | POLLOUT | POLLWRNORM;
 }
 
 /**
